import numpy as np
import ImageRecognitionSettings as settings
import faiss
from shutil import copyfile
import logging
import threading
import glob
import fileinput
import os
import GFUtilities

class FAISSSearchEngine:

    def __init__(self):
        #self.dataset = h5py.File(settings.dataset_file, 'r')['rmac'][...]

        #np.save('/media/Data/data/beni_culturali/deploy/dataset', self.dataset)
        #self.descs = np.load(settings.DATASET)
        #self.desc1 = np.load(settings.DATASET1)
        #self.desc2 = np.load(settings.DATASET2)

        #self.descs = (self.desc1 + self.desc2) / 2
        #self.descs /= np.linalg.norm(self.descs, axis=1, keepdims=True)
        #self.ids = np.loadtxt(settings.DATASET_IDS, dtype=str).tolist()
        #self.ids = np.loadtxt(fileinput.input(sorted(glob.glob(settings.DATASET_GF_FOLDER + '/*.ids'))))
        self.lock = threading.Lock()
        self.init_faiss_index()


    def init_faiss_index(self):
        # create an index with inner product similarity
        dim = 2048  # dimensionality of the features
        metric = faiss.METRIC_INNER_PRODUCT
        self.index = faiss.index_factory(dim, 'Flat', metric)
        self.descs = np.load(settings.DATASET)
        self.ids = np.loadtxt(settings.DATASET_IDS, dtype=str).tolist()
        self.index.add(self.descs)

        for desc_file in sorted(glob.glob(settings.DATASET_GF_FOLDER + '/**/*.dat', recursive=True)):
            # add the vectors to the index
            img_id = os.path.splitext(os.path.basename(desc_file))[0]
            img_desc = GFUtilities.unpickle_img_gf(settings.DATASET_GF_FOLDER, img_id)
            self.ids.append(img_id)
            self.index.add(img_desc)  # my_database is a numpy array of shape N x dim, where N is the number of vectors to index


    def get_id(self, idx):
        return self.ids[idx]

    def get_indexed_ids(self):
        return self.ids

    def exists(self, doc_id):
        return doc_id in self.ids

    def add(self, desc, doc_id):
        try:
            self.lock.acquire()
           # self.ids.append(doc_id)
           # self.descs = np.vstack((self.descs, desc))
            GFUtilities.pickle_img_gf(settings.DATASET_GF_FOLDER, doc_id, desc)
        finally:
            self.lock.release()

    def remove(self, doc_id):
        try:
            self.lock.acquire()
            if not GFUtilities.delete_img_gf( settings.DATASET_GF_FOLDER, doc_id):
                try:
                    idx = self.ids.index(doc_id)
                    del self.ids[idx]
                    self.descs = np.delete(self.descs, idx, axis=0)
                    descs_file = settings.DATASET
                    ids_file = settings.DATASET_IDS
                    np.save(descs_file, self.descs)
                    np.savetxt(ids_file, self.ids, fmt='%s')
                except ValueError as e:
                    logging.error('Error, unable to retrieve and delete ' + doc_id)
                    logging.error(e)
        finally:
            self.lock.release()

    def search_by_id(self, query_id, k=10):
        query_idx = self.ids.index(query_id)
        return self.search_by_img(self.descs[query_idx], k)

    def search_by_img(self, query, k=10):
        print('----------query features-------')
        logging.info(query)
        queries = np.reshape(query, (-1, 2048))
        logging.debug(queries)
        scores, indexes = self.index.search(queries, k)
        #dot_product = np.dot(self.descs, query)
        #idx = dot_product.argsort()[::-1][:k]
        res = []
        for (i,j) in zip(indexes[0], scores[0]):
            res.append((self.ids[i], round(float(j), 3)))
        return res

    def save(self, is_backup=False):
        descs_file = settings.DATASET
        ids_file = settings.DATASET_IDS
        try:
            self.lock.acquire()
            """if is_backup:
                descs_file_backup = descs_file +'.bak'
                ids_file_backup = ids_file + '.bak'
                copyfile(descs_file, descs_file_backup)
                copyfile(ids_file, ids_file_backup)
                logging.info('Backup features created')

            np.save(descs_file, self.descs)
            np.savetxt(ids_file, self.ids, fmt='%s')
            logging.info('Storing features done')"""
            self.init_faiss_index()
        except Exception as e:
            logging.error('Error, unable to storing features')
            logging.error(e)
        finally:
            self.lock.release()
