from flask import Flask, request, redirect, url_for, flash, render_template, send_from_directory, abort
from random import randint
import cv2
import io
import numpy as np
import json

import urllib

#from BeniCulturaliSearcher import BeniCulturaliSearcher
from Searcher import Searcher
import ImageRecognitionSettings as settings
import uuid
import os, os.path
import tornado.wsgi
import tornado.httpserver
import argparse
import logging
import base64


app = Flask(__name__)


@app.route('/bcir/')
def api_root():
    print('index_with_randoms.html')
    random_ids = []
    for i in range(0, 15):
        random_ids.append(searcher.get_id(randint(0, 600)))
    return render_template('index_with_randoms.html', random_ids=random_ids)


def url_to_file(url):
    dest_file = uuid.uuid4().hex + ".png"
    dest_path = settings.logs + "/" + dest_file
    req = urllib.request.Request(
        url,
        data=None,
        headers={
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36'
        }
    )
    resp = urllib.request.urlopen(req)
    image = np.asarray(bytearray(resp.read()), dtype="uint8")
    decoded = cv2.imdecode(image, cv2.IMREAD_COLOR)
    cv2.imwrite(dest_path, decoded)
    #im = Image.fromarray(image)
    #im.save(dest_path)
    return dest_path


def post_to_file(image):
    dest_file = uuid.uuid4().hex + ".png"
    dest_path = settings.logs + "/" + dest_file
    image.save(dest_path)
    return  dest_path


def get_res(results, query_url=None):
    if query_url is not None:
        return render_template('search.html', results=results, query_url=query_url)
    json_res = json.dumps(results)
    return json_res


@app.route('/bcir/searchById')
def search_by_id():
    id = request.args.get('id')
    k, threshold, search_deep_level = get_parameters(request.args.get("k"), request.args.get("threshold"),
                                                     request.args.get("searchDeepLevel"))
    results = searcher.search_by_id(id, k, threshold, search_deep_level)
    query_url = None
    if request.args.get("tohtml") is not None:
        query_url = id + ".jpg"
    return get_res(results, query_url)


@app.route('/bcir/searchByImg', methods=['POST'])
def search_by_img():
    if 'image' not in request.files:
        flash('No file part')
        return redirect(request.url)

    file = request.files['image']
    img_file = post_to_file(file)

    k, threshold, search_deep_level = get_parameters(request.form.get("k"), request.form.get("threshold"), request.form.get("searchDeepLevel"))

    #dest_file = uuid.uuid4().hex + ".jpg"
    #dest_path = settings.logs + "/" + dest_file
    #file.save(dest_path)
    #files = {'image': (dest_file, open(dest_path, 'rb'))}
    #r = requests.post(settings.rmac_service, files=files)
    #results = search_engine.search_by_img(np.array(r.json()), settings.k)
    results = searcher.search_by_img(img_file, k, threshold, search_deep_level)
    query_url = None
    if request.form.get("tohtml") is not None:
        query_url = ""
    return get_res(results, query_url)


@app.route('/bcir/searchByImgB64', methods=['POST'])
def search_by_img_base64():
    image = request.form.get('image')
    if image:
        img_file = base64_to_file(image)
    else:
        flash('No img sent')
        return redirect(request.url)

    k, threshold, search_deep_level = get_parameters(request.form.get("k"), request.form.get("threshold"), request.form.get("searchDeepLevel"))

    results = searcher.search_by_img(img_file, k, threshold, search_deep_level)
    query_url = None
    if request.form.get("tohtml") is not None:
        query_url = ""
    return get_res(results, query_url)


def base64_to_file(image_base64):
    ext = ".png"
    dest_file = uuid.uuid4().hex + ext
    dest_path = settings.logs + "/" + dest_file
    with open(dest_path, "wb") as image_file:
        byte_content = base64.b64decode(image_base64)
        image_file.write(byte_content)
    return  dest_path

@app.route('/bcir/searchByURL')
def search_by_url():
    url = request.args.get('url')

    img_file = url_to_file(url)

    k, threshold, search_deep_level = get_parameters(request.args.get("k"), request.args.get("threshold"),
                                                     request.args.get("searchDeepLevel"))
   # query = cv2.imdecode(image, cv2.IMREAD_COLOR)
   # dest_file = uuid.uuid4().hex + ".jpg"
   # dest_path = settings.logs + "/" + dest_file
   # cv2.imwrite(dest_path, query)
   # files = {'image': open(dest_path, 'rb')}
   # r = requests.post(settings.rmac_service, files=files)
   # results = search_engine.search_by_img(np.array(r.json()), settings.k)
    results = searcher.search_by_img(img_file, k, threshold, search_deep_level)
    query_url = None
    if request.args.get("tohtml") is not None:
        query_url = url
    return get_res(results, query_url)


def get_parameters(k, threshold, search_deep_level):
    try:
        threshold = float(threshold)
    except Exception as e:
        logging.error(e)
        threshold = settings.SEARCH_THRESHOLD
        logging.error('Setting default threshold value to ' + str(threshold))
    try:
        k = int(k)
    except Exception as e:
        logging.error(e)
        k = settings.k
        logging.error('Setting default k value to ' + str(k))

    try:
        search_deep_level = int(search_deep_level)
    except Exception as e:
        logging.error(e)
        search_deep_level = settings.SEARCH_DEEP_LEVEL
        logging.error('Setting default deep level to ' + str(search_deep_level))

    return k, threshold, search_deep_level


@app.route('/bcir/getIds')
def get_indexed_ids():
    json_ids = json.dumps(searcher.get_indexed_ids())
    return json_ids


@app.route('/bcir/exists')
def exists():
    doc_id = request.args.get('id')
    return json.dumps(searcher.exists(doc_id))


@app.route('/bcir/addImg', methods=['POST'])
def add_img():
    if 'image' not in request.files:
        flash('No file part')
        return redirect(request.url)
    try:
        file = request.files['image']
        doc_id = request.files['image'].filename
        doc_id, _ = os.path.splitext(doc_id)
        img_file = post_to_file(file)
        searcher.add(img_file, doc_id)
        json_res = json.dumps("done")
        return json_res
    except Exception as e:
        logging.error('Unable to add ' +  doc_id + 'to the index')
        logging.error(e)
        abort(500)


@app.route('/bcir/rmImg')
def remove_img():
    try:
        doc_id = request.args.get('id')
        searcher.remove(doc_id)
        json_res = json.dumps("done")
        return json_res
    except Exception as e:
        logging.error('Unable to remove ' + doc_id + 'to the index')
        logging.error(e)
        abort(500)

@app.route('/bcir/<path:filename>')
def download_file(filename):
    logging.debug(filename)
    values = filename.split('/')
    logging.debug(values)

    return send_from_directory(settings.img_folder, filename, as_attachment=False)

@app.route('/bcir/queries/<path:filename>')
def queries(filename):
    logging.debug(filename)
    values = filename.split('/')
    folder = values[0]
    name = values[1]
    logging.debug(folder)
    logging.debug(name)

    return send_from_directory(settings.working_folder + '/' + folder, name, as_attachment=False)

"""
if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Reading configuration file')
    parser.add_argument('conf', type=str, help='Configuration file path')

    args = parser.parse_args()
    settings.load_setting(args.conf)
    global searcher
    searcher = BeniCulturaliSearcher()

    #app.run(host='0.0.0.0', port=8090, ssl_context='adhoc')
    app.run(host='0.0.0.0', port=settings.port)

  #  app.run(host='0.0.0.0', port=settings.port)
"""

def start_tornado(app, port=8190):
    http_server = tornado.httpserver.HTTPServer(tornado.wsgi.WSGIContainer(app))
    http_server.listen(port)
    app.logger.info("Tornado server starting on port {}".format(port))
    tornado.ioloop.IOLoop.instance().start()


def start_from_terminal(app):
    parser = argparse.ArgumentParser(description='Reading configuration file')
    parser.add_argument('conf', type=str, help='Configuration file path')

    args = parser.parse_args()
    settings.load_setting(args.conf)
    global searcher
    searcher = Searcher()

    #if args.debug:
    #    app.run(debug=True, host='0.0.0.0', port=settings.port)
#    else:
    #start_tornado(app, settings.port)

    app.run(debug=False, host='0.0.0.0', port=settings.port)


if __name__ == '__main__':
    start_from_terminal(app)

