import glob, os
from PIL import Image
import argparse
from pathlib import Path
import tqdm


max_size = 1050


def resize_img(src, dest):
    if not os.path.isdir(dest):
        os.mkdir(dest)
    paths = Path(src).rglob('*.*')
    paths_list = list(paths)

    for path in tqdm.tqdm(paths_list):
        #print(path.name)
        id, _ = os.path.splitext(path.name)

        outfile = os.path.join(dest, id + ".jpg")
        try:
            im = Image.open(path)
            if im.mode in ("RGBA", "P"):
                im = im.convert("RGB")
            width, height = im.size
            if width > max_size or height > max_size:
                im.thumbnail((max_size, max_size), Image.ANTIALIAS)
            im.save(outfile, "JPEG")
        except IOError:
            print("cannot create thumbnail for '%s'" % path)
            pass


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Image resizing')
    parser.add_argument('src', type=str, help='images source folder path')
    parser.add_argument('dest', type=str, help='images dest folder path')

    #args = parser.parse_args()
    #resize_img(args.src, args.dest)
    resize_img('/media/Data/data/test/gem/img/originals/export_Immagini_SitoPubblico', '/media/Data/data/test/gem/img/export_Immagini_SitoPubblico_resized')
    #resize_img('/media/Data/data/test/gem/img/originals/ImmaginiComparazioni', '/media/Data/data/test/gem/img/ImmaginiComparazioni_resized')
