import cv2
import numpy as np
import pickle as pickle
import os
from line_profiler_pycharm import profile

def resize(max_side, img):
    if img.shape[1] > img.shape[0]:
        r = max_side / img.shape[1]
        dim = (max_side, int(img.shape[0] * r))
    else:
        r = max_side / img.shape[0]
        dim = (int(img.shape[1] * r), max_side)

    # perform the actual resizing of the image and show it
    resized = cv2.resize(img, dim, interpolation=cv2.INTER_AREA)
    return resized


def pickle_keypoints(keypoints, descriptors):
    i = 0
    temp_array = []
    for point in keypoints:
        temp = (point.pt, point.size, point.angle, point.response, point.octave,
        point.class_id, descriptors[i])
        i += 1
        temp_array.append(temp)
    return temp_array


def serialize_object(obj):
    return pickle.dumps(obj)


def deserialize_object(serialized_obj):
    return pickle.loads(serialized_obj)


def serializeV1(keypoints, descriptors):
    temp_array = []
    for point in keypoints:
        kp = [point.pt, point.size, point.angle, point.response, point.octave, point.class_id]
        temp_array.append(kp)
    return temp_array, descriptors


def serialize(keypoints, descriptors):
    pts = np.float32([keypoints[i].pt for i in range(0, len(keypoints))])
    return pts, descriptors

def deserialize(ser_kp, ser_des):
    keypoints = []
    #data_list = array.tolist()
    for point in ser_kp:
        temp_feature = cv2.KeyPoint(x=point[0][0],y=point[0][1], size=point[1], angle=point[2], response=point[3], octave=point[4], class_id=point[5])
        keypoints.append(temp_feature)
    return keypoints, ser_des


def deserializev1(ser_kp, ser_des):
    keypoints = []
    #data_list = array.tolist()
    for point in ser_kp:
        temp_feature = cv2.KeyPoint(x=point[0][0],y=point[0][1], size=point[1], angle=point[2], response=point[3], octave=point[4], class_id=point[5])
        keypoints.append(temp_feature)
    return keypoints, ser_des

def pickle_img_lf(dest, id, keypoints, descriptors):
    dest_folder_name = id[0:3]
    filename = id + '.dat'
    dest_folder_path = os.path.join(dest, dest_folder_name)
    if (not os.path.exists(dest_folder_path)):
        os.mkdir(dest_folder_path)
    dest_path = os.path.join(dest_folder_path, filename)
    kps, des = serialize(keypoints, descriptors)
    pickle.dump([kps, des], open(dest_path, 'wb'))

def delete_img_lf(dest, id):
    dest_folder_name = id[0:3]
    filename = id + '.dat'
    dest_folder_path = os.path.join(dest, dest_folder_name)
    dest_path = os.path.join(dest_folder_path, filename)
    if os.path.exists(dest_path):
        os.remove(dest_path)

@profile
def unpickle_img_lf(lf_path, id):
    dest_folder_name = id[0:3]
    filename = id + '.dat'
    dest_folder_path = os.path.join(lf_path, dest_folder_name)
    dest_path = os.path.join(dest_folder_path, filename)
    kps,  des = pickle.load((open(dest_path, "rb")))
    return kps,  des


@profile
def loadz_img_lf(lf_path, id):
    dest_folder_name = id[0:3]
    filename = id + '.dat.npz'
    dest_folder_path = os.path.join(lf_path, dest_folder_name)
    dest_path = os.path.join(dest_folder_path, filename)
    data = np.load(dest_path, allow_pickle=False)
    kps = data.f.kps
    des = data.f.des
    #kps = data['kps']
    #des = data['des']
    #kp, desc = deserialize(data['kps'], data['des'])
    return kps,  des


def savez_img_lf(dest, id, keypoints, descriptors):
    dest_folder_name = id[0:3]
    filename = id + '.dat'
    dest_folder_path = os.path.join(dest, dest_folder_name)
    if (not os.path.exists(dest_folder_path)):
        os.mkdir(dest_folder_path)
    dest_path = os.path.join(dest_folder_path, filename)
    kps, des = serialize(keypoints, descriptors)
    #np.savez(dest_path, data)
    np.savez(dest_path, kps=kps, des=des)


@profile
def loadz_img_lf(lf_path, id):
    dest_folder_name = id[0:3]
    filename = id + '.dat.npz'
    dest_folder_path = os.path.join(lf_path, dest_folder_name)
    dest_path = os.path.join(dest_folder_path, filename)
    data = np.load(dest_path, allow_pickle=False)
    kps = data.f.kps
    des = data.f.des
    #kps = data['kps']
    #des = data['des']
    #kp, desc = deserialize(data['kps'], data['des'])
    return kps,  des


def unpickle_keypoints(array):
    keypoints = []
    descriptors = []
    data_list = array.tolist()
    for point in array:
        temp_feature = cv2.KeyPoint(x=point[0][0],y=point[0][1], size=point[1], angle=point[2], response=point[3], octave=point[4], class_id=point[5])
        temp_descriptor = point[6]
        keypoints.append(temp_feature)
        descriptors.append(temp_descriptor)
    return keypoints, np.array(descriptors)


def load(lf_path):
    print('loading LF dataset ' + lf_path)
    ser_dataset = pickle.load(open(lf_path, "rb"))
    lf_dataset = []
    for item in ser_dataset:
        kp, desc = unpickle_keypoints(item)
        lf_dataset.append((kp, desc))
    return lf_dataset


def save(lf_data, lf_path):
    data = []
    for lf in lf_data:
        data.append(pickle_keypoints(lf[0], lf[1]))
    pickle.dump(data, open(lf_path, 'wb'))


def save_img_lf(dest, id, keypoints, descriptors):
    dest_folder_name = id[0:3]
    filename = id + '.dat'
    dest_folder_path = os.path.join(dest, dest_folder_name)
    if (not os.path.exists(dest_folder_path)):
        os.mkdir(dest_folder_path)
    dest_path = os.path.join(dest_folder_path, filename)
    data = pickle_keypoints(keypoints, descriptors)
    pickle.dump(data, open(dest_path, 'wb'))


def load_img_lf(lf_path, id):
    dest_folder_name = id[0:3]
    filename = id + '.dat'
    dest_folder_path = os.path.join(lf_path, dest_folder_name)
    dest_path = os.path.join(dest_folder_path, filename)
    data = pickle.load(open(dest_path, "rb"))
    kp, desc = unpickle_keypoints(data)
    return (kp, desc)


def load_img_lf_GPU(lf_path, id):
    dest_folder_name = id[0:3]
    filename = id + '.dat'
    dest_folder_path = os.path.join(lf_path, dest_folder_name)
    dest_path = os.path.join(dest_folder_path, filename)
    data = pickle.load(open(dest_path, "rb"))
    kp, desc = unpickle_keypoints(data)

    data_gpu_mat = cv2.cuda_GpuMat(np.zeros((1500,), dtype=int))
    if len(desc) > 0:
        data_gpu_mat = cv2.cuda_GpuMat(desc)
    desc = data_gpu_mat

    return (kp, desc)

